@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -21,11 +21,20 @@
 
 #include <drivers/atmodem/atutil.h>
 
-extern void ifx_voicecall_init();
-extern void ifx_voicecall_exit();
+extern void ifx_voicecall_init(void);
+extern void ifx_voicecall_exit(void);
 
-extern void ifx_radio_settings_init();
-extern void ifx_radio_settings_exit();
+extern void ifx_audio_settings_init(void);
+extern void ifx_audio_settings_exit(void);
 
-extern void ifx_stk_init();
-extern void ifx_stk_exit();
+extern void ifx_radio_settings_init(void);
+extern void ifx_radio_settings_exit(void);
+
+extern void ifx_gprs_context_init(void);
+extern void ifx_gprs_context_exit(void);
+
+extern void ifx_stk_init(void);
+extern void ifx_stk_exit(void);
+
+extern void ifx_ctm_init(void);
+extern void ifx_ctm_exit(void);
\ No newline at end of file
