@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *  Copyright (C) 2010  ProFUSION embedded systems
  *  Copyright (C) 2010  Gustavo F. Padovan <gustavo@padovan.org>
  *
@@ -33,7 +33,6 @@
 #include <gdbus.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
-#include <ofono/dbus.h>
 #include <ofono/plugin.h>
 #include <ofono/log.h>
 
@@ -90,13 +89,13 @@ void bluetooth_create_path(const char *dev_addr, const char *adapter_addr,
 }
 
 int bluetooth_send_with_reply(const char *path, const char *interface,
-				const char *method,
+				const char *method, DBusPendingCall **call,
 				DBusPendingCallNotifyFunction cb,
 				void *user_data, DBusFreeFunction free_func,
 				int timeout, int type, ...)
 {
 	DBusMessage *msg;
-	DBusPendingCall *call;
+	DBusPendingCall *c;
 	va_list args;
 	int err;
 
@@ -121,14 +120,18 @@ int bluetooth_send_with_reply(const char *path, const char *interface,
 	if (timeout > 0)
 		timeout *= 1000;
 
-	if (!dbus_connection_send_with_reply(connection, msg, &call, timeout)) {
+	if (!dbus_connection_send_with_reply(connection, msg, &c, timeout)) {
 		ofono_error("Sending %s failed", method);
 		err = -EIO;
 		goto fail;
 	}
 
-	dbus_pending_call_set_notify(call, cb, user_data, free_func);
-	dbus_pending_call_unref(call);
+	if (call != NULL)
+		*call = c;
+
+	dbus_pending_call_set_notify(c, cb, user_data, free_func);
+	dbus_pending_call_unref(c);
+
 	dbus_message_unref(msg);
 
 	return 0;
@@ -226,9 +229,9 @@ done:
 	g_slist_free(prop_handlers);
 }
 
-static void has_uuid(DBusMessageIter *array, gpointer user_data)
+static void parse_uuids(DBusMessageIter *array, gpointer user_data)
 {
-	gboolean *profiles = user_data;
+	GSList **uuids = user_data;
 	DBusMessageIter value;
 
 	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
@@ -241,8 +244,7 @@ static void has_uuid(DBusMessageIter *array, gpointer user_data)
 
 		dbus_message_iter_get_basic(&value, &uuid);
 
-		if (!strcasecmp(uuid, HFP_AG_UUID))
-			*profiles |= HFP_AG;
+		*uuids = g_slist_prepend(*uuids, (char *) uuid);
 
 		dbus_message_iter_next(&value);
 	}
@@ -259,17 +261,38 @@ static void parse_string(DBusMessageIter *iter, gpointer user_data)
 	dbus_message_iter_get_basic(iter, str);
 }
 
+static void bluetooth_probe(GSList *uuids, const char *path,
+				const char *device, const char *adapter,
+				const char *alias)
+{
+	for (; uuids; uuids = uuids->next) {
+		struct bluetooth_profile *driver;
+		const char *uuid = uuids->data;
+		int err;
+
+		driver = g_hash_table_lookup(uuid_hash, uuid);
+		if (driver == NULL)
+			continue;
+
+		err = driver->probe(path, device, adapter, alias);
+		if (err == 0)
+			continue;
+
+		ofono_error("%s probe: %s (%d)", driver->name, strerror(-err),
+									-err);
+	}
+}
+
 static void device_properties_cb(DBusPendingCall *call, gpointer user_data)
 {
 	DBusMessage *reply;
-	int have_uuid = 0;
 	const char *path = user_data;
 	const char *adapter = NULL;
 	const char *adapter_addr = NULL;
 	const char *device_addr = NULL;
 	const char *alias = NULL;
-	struct bluetooth_profile *profile;
 	struct DBusError derr;
+	GSList *uuids = NULL;
 
 	reply = dbus_pending_call_steal_reply(call);
 
@@ -284,7 +307,7 @@ static void device_properties_cb(DBusPendingCall *call, gpointer user_data)
 
 	DBG("");
 
-	bluetooth_parse_properties(reply, "UUIDs", has_uuid, &have_uuid,
+	bluetooth_parse_properties(reply, "UUIDs", parse_uuids, &uuids,
 				"Adapter", parse_string, &adapter,
 				"Address", parse_string, &device_addr,
 				"Alias", parse_string, &alias, NULL);
@@ -293,15 +316,13 @@ static void device_properties_cb(DBusPendingCall *call, gpointer user_data)
 		adapter_addr = g_hash_table_lookup(adapter_address_hash,
 							adapter);
 
-	if ((have_uuid & HFP_AG) && device_addr && adapter_addr) {
-		profile = g_hash_table_lookup(uuid_hash, HFP_AG_UUID);
-		if (profile == NULL || profile->create == NULL)
-			goto done;
+	if (!device_addr || !adapter_addr)
+		goto done;
 
-		profile->create(path, device_addr, adapter_addr, alias);
-	}
+	bluetooth_probe(uuids, path, device_addr, adapter_addr, alias);
 
 done:
+	g_slist_free(uuids);
 	dbus_message_unref(reply);
 }
 
@@ -342,7 +363,7 @@ static gboolean property_changed(DBusConnection *connection, DBusMessage *msg,
 
 	dbus_message_iter_get_basic(&iter, &property);
 	if (g_str_equal(property, "UUIDs") == TRUE) {
-		int profiles = 0;
+		GSList *uuids = NULL;
 		const char *path = dbus_message_get_path(msg);
 		DBusMessageIter variant;
 
@@ -354,17 +375,17 @@ static gboolean property_changed(DBusConnection *connection, DBusMessage *msg,
 
 		dbus_message_iter_recurse(&iter, &variant);
 
-		has_uuid(&variant, &profiles);
+		parse_uuids(&variant, &uuids);
 
 		/* We need the full set of properties to be able to create
 		 * the modem properly, including Adapter and Alias, so
 		 * refetch everything again
 		 */
-		if (profiles)
+		if (uuids)
 			bluetooth_send_with_reply(path, BLUEZ_DEVICE_INTERFACE,
-					"GetProperties", device_properties_cb,
-					g_strdup(path), g_free, -1,
-					DBUS_TYPE_INVALID);
+					"GetProperties", NULL,
+					device_properties_cb, g_strdup(path),
+					g_free, -1, DBUS_TYPE_INVALID);
 	} else if (g_str_equal(property, "Alias") == TRUE) {
 		const char *path = dbus_message_get_path(msg);
 		struct bluetooth_profile *profile;
@@ -429,9 +450,9 @@ static void adapter_properties_cb(DBusPendingCall *call, gpointer user_data)
 		const char *device = l->data;
 
 		bluetooth_send_with_reply(device, BLUEZ_DEVICE_INTERFACE,
-					"GetProperties", device_properties_cb,
-					g_strdup(device), g_free, -1,
-					DBUS_TYPE_INVALID);
+					"GetProperties", NULL,
+					device_properties_cb, g_strdup(device),
+					g_free, -1, DBUS_TYPE_INVALID);
 	}
 
 done:
@@ -443,7 +464,7 @@ static void get_adapter_properties(const char *path, const char *handle,
 						gpointer user_data)
 {
 	bluetooth_send_with_reply(path, BLUEZ_ADAPTER_INTERFACE,
-			"GetProperties", adapter_properties_cb,
+			"GetProperties", NULL, adapter_properties_cb,
 			g_strdup(path), g_free, -1, DBUS_TYPE_INVALID);
 }
 
@@ -590,7 +611,7 @@ static void new_connection(GIOChannel *io, gpointer user_data)
 	addr = raddress;
 
 	if (bluetooth_send_with_reply(path, BLUEZ_SERVICE_INTERFACE,
-					"RequestAuthorization",
+					"RequestAuthorization", NULL,
 					auth_cb, cbd, cb_data_destroy,
 					TIMEOUT, DBUS_TYPE_STRING, &addr,
 					DBUS_TYPE_UINT32, &server->handle,
@@ -646,7 +667,7 @@ static void add_record(gpointer data, gpointer user_data)
 
 	bluetooth_send_with_reply(adapter_any_path,
 					BLUEZ_SERVICE_INTERFACE, "AddRecord",
-					add_record_cb, server, NULL, -1,
+					NULL, add_record_cb, server, NULL, -1,
 					DBUS_TYPE_STRING, &server->sdp_record,
 					DBUS_TYPE_INVALID);
 }
@@ -686,20 +707,44 @@ static gboolean adapter_added(DBusConnection *connection, DBusMessage *message,
 				DBUS_TYPE_INVALID);
 
 	bluetooth_send_with_reply(path, BLUEZ_ADAPTER_INTERFACE,
-			"GetProperties", adapter_properties_cb, g_strdup(path),
-			g_free, -1, DBUS_TYPE_INVALID);
+			"GetProperties", NULL, adapter_properties_cb,
+			g_strdup(path), g_free, -1, DBUS_TYPE_INVALID);
 
 	return TRUE;
 }
 
+static void bluetooth_remove(gpointer key, gpointer value, gpointer user_data)
+{
+	struct bluetooth_profile *profile = value;
+
+	profile->remove(user_data);
+}
+
 static gboolean adapter_removed(DBusConnection *connection,
 				DBusMessage *message, void *user_data)
 {
 	const char *path;
 
 	if (dbus_message_get_args(message, NULL, DBUS_TYPE_OBJECT_PATH, &path,
-				DBUS_TYPE_INVALID) == TRUE)
-		g_hash_table_remove(adapter_address_hash, path);
+				DBUS_TYPE_INVALID) == FALSE)
+		return FALSE;
+
+	g_hash_table_foreach(uuid_hash, bluetooth_remove, (gpointer) path);
+	g_hash_table_remove(adapter_address_hash, path);
+
+	return TRUE;
+}
+
+static gboolean device_removed(DBusConnection *connection,
+				DBusMessage *message, void *user_data)
+{
+	const char *path;
+
+	if (dbus_message_get_args(message, NULL, DBUS_TYPE_OBJECT_PATH, &path,
+				DBUS_TYPE_INVALID) == FALSE)
+		return FALSE;
+
+	g_hash_table_foreach(uuid_hash, bluetooth_remove, (gpointer) path);
 
 	return TRUE;
 }
@@ -724,7 +769,7 @@ static void parse_adapters(DBusMessageIter *array, gpointer user_data)
 		DBG("Calling GetProperties on %s", path);
 
 		bluetooth_send_with_reply(path, BLUEZ_ADAPTER_INTERFACE,
-				"GetProperties", adapter_properties_cb,
+				"GetProperties", NULL, adapter_properties_cb,
 				g_strdup(path), g_free, -1, DBUS_TYPE_INVALID);
 
 		dbus_message_iter_next(&value);
@@ -756,22 +801,14 @@ done:
 	dbus_message_unref(reply);
 }
 
-static void bluetooth_remove_all_modem(gpointer key, gpointer value,
-					gpointer user_data)
-{
-	struct bluetooth_profile *profile = value;
-
-	profile->remove_all();
-}
-
 static void bluetooth_connect(DBusConnection *connection, void *user_data)
 {
 	bluetooth_send_with_reply("/", BLUEZ_MANAGER_INTERFACE, "GetProperties",
-				manager_properties_cb, NULL, NULL, -1,
+				NULL, manager_properties_cb, NULL, NULL, -1,
 				DBUS_TYPE_INVALID);
 
 	bluetooth_send_with_reply("/", BLUEZ_MANAGER_INTERFACE, "FindAdapter",
-				find_adapter_cb, NULL, NULL, -1,
+				NULL, find_adapter_cb, NULL, NULL, -1,
 				DBUS_TYPE_STRING, &adapter_any_name,
 				DBUS_TYPE_INVALID);
 }
@@ -781,7 +818,7 @@ static void bluetooth_disconnect(DBusConnection *connection, void *user_data)
 	if (uuid_hash == NULL)
 		return;
 
-	g_hash_table_foreach(uuid_hash, bluetooth_remove_all_modem, NULL);
+	g_hash_table_foreach(uuid_hash, bluetooth_remove, NULL);
 
 	g_slist_foreach(server_list, (GFunc) remove_service_handle, NULL);
 }
@@ -789,6 +826,7 @@ static void bluetooth_disconnect(DBusConnection *connection, void *user_data)
 static guint bluetooth_watch;
 static guint adapter_added_watch;
 static guint adapter_removed_watch;
+static guint device_removed_watch;
 static guint property_watch;
 
 static void bluetooth_ref(void)
@@ -812,6 +850,11 @@ static void bluetooth_ref(void)
 						"AdapterRemoved",
 						adapter_removed, NULL, NULL);
 
+	device_removed_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						BLUEZ_ADAPTER_INTERFACE,
+						"DeviceRemoved",
+						device_removed, NULL, NULL);
+
 	property_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
 						BLUEZ_DEVICE_INTERFACE,
 						"PropertyChanged",
@@ -857,6 +900,12 @@ static void bluetooth_unref(void)
 	g_hash_table_destroy(adapter_address_hash);
 }
 
+void bluetooth_get_properties()
+{
+	g_hash_table_foreach(adapter_address_hash,
+				(GHFunc) get_adapter_properties, NULL);
+}
+
 int bluetooth_register_uuid(const char *uuid, struct bluetooth_profile *profile)
 {
 	bluetooth_ref();
