@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -1489,8 +1489,12 @@ static void registration_status_cb(const struct ofono_error *error,
 	DBG("%s error %d status %d", __ofono_atom_get_path(gprs->atom),
 		error->type, status);
 
+	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+
 	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
 		ofono_gprs_status_notify(gprs, status);
+	else
+		gprs_attached_update(gprs);
 
 	if (gprs->flags & GPRS_FLAG_RECHECK) {
 		gprs->flags &= ~GPRS_FLAG_RECHECK;
@@ -1504,23 +1508,20 @@ static void gprs_attach_callback(const struct ofono_error *error, void *data)
 
 	DBG("%s error = %d", __ofono_atom_get_path(gprs->atom), error->type);
 
-	gprs->flags &= ~GPRS_FLAG_ATTACHING;
-
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR)
 		gprs->driver_attached = !gprs->driver_attached;
 
-	if (gprs->driver->attached_status) {
-		gprs->driver->attached_status(gprs, registration_status_cb,
-						gprs);
+	if (gprs->driver->attached_status == NULL) {
+		struct ofono_error error;
+
+		error.type = OFONO_ERROR_TYPE_FAILURE;
+		error.error = 0;
+
+		registration_status_cb(&error, -1, gprs);
 		return;
 	}
 
-	gprs_attached_update(gprs);
-
-	if (gprs->flags & GPRS_FLAG_RECHECK) {
-		gprs->flags &= ~GPRS_FLAG_RECHECK;
-		gprs_netreg_update(gprs);
-	}
+	gprs->driver->attached_status(gprs, registration_status_cb, gprs);
 }
 
 static void gprs_netreg_removed(struct ofono_gprs *gprs)
@@ -2107,6 +2108,14 @@ void ofono_gprs_status_notify(struct ofono_gprs *gprs, int status)
 		return;
 	}
 
+	/*
+	 * If we're already taking action, e.g. attaching or detaching, then
+	 * ignore this notification for now, we will take appropriate action
+	 * after the set_attach operation has completed
+	 */
+	if (gprs->flags & GPRS_FLAG_ATTACHING)
+		return;
+
 	/* We registered without being powered */
 	if (gprs->powered == FALSE)
 		goto detach;
@@ -2824,7 +2833,7 @@ static void provision_context(const struct ofono_gprs_provision_data *ap,
 	if (ap == NULL)
 		return;
 
-	if (ap->name == NULL || strlen(ap->name) > MAX_CONTEXT_NAME_LENGTH)
+	if (ap->name && strlen(ap->name) > MAX_CONTEXT_NAME_LENGTH)
 		return;
 
 	if (ap->apn == NULL || strlen(ap->apn) > OFONO_GPRS_MAX_APN_LENGTH)
