@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -55,6 +55,7 @@ struct modem_data {
 	guint audio_changed_watch;
 
 	gboolean has_callmanager;
+	gboolean has_audiosettings;
 	gboolean is_huawei;
 	gint audio_users;
 	guint audio_watch;
@@ -92,6 +93,10 @@ static gboolean audio_receive(GIOChannel *channel,
 		return TRUE;
 
 	wlen = write(modem->dsp_out, buf, rlen);
+	if (wlen < 0) {
+		modem->audio_watch = 0;
+		return FALSE;
+	}
 
 	return TRUE;
 }
@@ -448,6 +453,8 @@ static void check_interfaces(struct modem_data *modem, DBusMessageIter *iter)
 		dbus_message_iter_next(&entry);
 	}
 
+	modem->has_audiosettings = has_audiosettings;
+
 	if (modem->has_callmanager == has_callmanager)
 		return;
 
